import Vue from 'vue';
import { createVuexAlong } from 'vuex-along';
import router from './router';
import store from './store';
// router.hook 白名单
var HOOK_KEYS = Object.freeze({
    beforeEach: true,
    beforeResolve: true,
    afterEach: true,
    onReady: true,
    onError: true
});
var activator = {
    start: function (modular) {
        var application = modular.getApplication();
        var vuexAlong = {};
        vuexAlong.name = "VAS:" + application.name + "-" + application.version;
        // 处理 vuex.modules
        var local = [];
        var session = [];
        var vuexModules = modular.getExtension('vuex.modules');
        for (var _i = 0, _a = Object.keys(vuexModules); _i < _a.length; _i++) {
            var key = _a[_i];
            var m = vuexModules[key];
            store.registerModule(key, m);
            if (m.storage) {
                if (m.storage === 'session') {
                    session.push(key);
                }
                else if (m.storage === 'local') {
                    local.push(key);
                }
            }
        }
        if (session.length > 0) {
            vuexAlong.session = {
                list: session
            };
        }
        if (local.length > 0) {
            vuexAlong.local = {
                list: local
            };
        }
        else {
            vuexAlong.justSession = true;
        }
        // vuex.plugin 的安装方法
        createVuexAlong(vuexAlong)(store);
        // 处理 vue.plugins
        var vuePlugins = modular.getExtensions('vue.plugins');
        if (vuePlugins !== null) {
            vuePlugins.forEach(function (plugin) { return Vue.use(plugin); });
        }
        // 处理 vue.options
        var vueOptions = modular.getExtensions('vue.options') || [];
        // 处理 vue.router.routes
        var routerConfigs = modular.getExtension('vue.router.routes');
        var routes = [];
        var parentRoutes = {};
        var unresolved = {};
        function registerParentRoutes(route) {
            // 有 children 属性的路由才可以作为父路由注册
            if (route.children !== undefined && Array.isArray(route.children)) {
                var name_1 = route.name;
                if (name_1 === undefined || name_1 === '') {
                    // tslint:disable-next-line:no-console
                    console.log("Error: \u8DEF\u7531\u540D\u79F0\u672A\u5B9A\u4E49\uFF0C" + JSON.stringify(route));
                    return;
                }
                var children_1 = route.children;
                parentRoutes[name_1] = children_1;
                if (unresolved[name_1]) {
                    // 加入后将未解决的该名称下子路由拷贝过来
                    unresolved[name_1].forEach(function (item) {
                        children_1.push(item);
                    });
                    delete unresolved[name_1];
                }
                route.children.forEach(function (item) {
                    registerParentRoutes(item);
                });
            }
        }
        if (routerConfigs !== null) {
            routerConfigs.forEach(function (config) {
                if (config && config.parent && config.routes) {
                    var parentName = config.parent;
                    var parent_1;
                    if (parentName === 'root') {
                        parent_1 = routes;
                    }
                    else {
                        parent_1 = parentRoutes[parentName];
                        if (parent_1 === undefined) {
                            // 暂存为未解决状态
                            unresolved[parentName] = unresolved[parentName] || [];
                            parent_1 = unresolved[parentName];
                        }
                    }
                    config.routes.forEach(function (item) {
                        // TODO 此处可以加入路由配置规则校验：路由名称未定义、路由缓存开启条件不满足……
                        parent_1.push(item);
                        registerParentRoutes(item);
                    });
                }
            });
            for (var _b = 0, _c = Object.keys(unresolved); _b < _c.length; _b++) {
                var key = _c[_b];
                // TODO 完善日志机制
                // tslint:disable-next-line:no-console
                console.log("Error: \u7236\u8DEF\u7531\u201C" + key + "\u201D\u4E0D\u5B58\u5728");
            }
            // 加入路由
            router.addRoutes(routes);
        }
        // 处理 vue.router.hooks
        var hooks = modular.getExtensions('vue.router.hooks');
        if (hooks !== null) {
            var register_1 = router;
            hooks.forEach(function (config) {
                for (var hook in config) {
                    if (HOOK_KEYS[hook] !== undefined) {
                        register_1[hook](config[hook]);
                    }
                }
            });
        }
        // 处理 vue.app
        var app = modular.getExtension('vue.app');
        var vm = new Vue({
            router: router,
            store: store,
            mixins: vueOptions,
            render: function (h) { return h(app.component); }
        }).$mount('#app');
        modular.setAttribute('vue.instance', vm);
        // start end
    }
};
export default activator;
